library split_view;

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

/// SplitView
class SplitView extends StatefulWidget {
  final Widget view1;
  final Widget view2;
  final SplitViewMode viewMode;
  final double gripSize;
  final double initialWeight;
  final Color gripColor;
  final double positionLimit;

  SplitView({
    @required this.view1,
    @required this.view2,
    @required this.viewMode,
    this.gripSize = 12.0,
    this.initialWeight = 0.5,
    this.gripColor = Colors.grey,
    this.positionLimit = 20.0,
  });

  @override
  State createState() => _SplitViewState();
}

class _SplitViewState extends State<SplitView> {
  double defaultWeight;
  ValueNotifier<double> weight;

  @override
  void initState() {
    super.initState();
    this.defaultWeight = widget.initialWeight;
  }

  @override
  Widget build(BuildContext context) {
    weight = ValueNotifier(defaultWeight);

    return LayoutBuilder(
      builder: (context, constraints) {
        return ValueListenableBuilder<double>(
          valueListenable: weight,
          builder: (_, w, __) {
            if (widget.viewMode == SplitViewMode.Vertical) {
              return _buildVerticalView(context, constraints, w);
            } else {
              return _buildHorizontalView(context, constraints, w);
            }
          },
        );
      },
    );
  }

  Stack _buildVerticalView(BuildContext context, BoxConstraints constraints, double w) {
    double top = constraints.maxHeight * w;
    double bottom = constraints.maxHeight * (1.0 - w);

    return Stack(
      children: <Widget>[
        Positioned(
          top: 0,
          left: 0,
          right: 0,
          bottom: bottom,
          child: widget.view1,
        ),
        Positioned(
          top: top,
          left: 0,
          right: 0,
          bottom: 0,
          child: widget.view2,
        ),
        Positioned(
          top: top - widget.gripSize / 2.0,
          left: 0,
          right: 0,
          bottom: bottom - widget.gripSize / 2.0,
          child: GestureDetector(
            behavior: HitTestBehavior.translucent,
            onVerticalDragUpdate: (detail) {
              final RenderBox container = context.findRenderObject() as RenderBox;
              final pos = container.globalToLocal(detail.globalPosition);
              if (pos.dy > widget.positionLimit &&
                  pos.dy < (container.size.height - widget.positionLimit)) {
                weight.value = pos.dy / container.size.height;
              }
            },
            child: Container(color: widget.gripColor),
          ),
        ),
      ],
    );
  }

  Widget _buildHorizontalView(BuildContext context, BoxConstraints constraints, double w) {
    final double left = constraints.maxWidth * w;
    final double right = constraints.maxWidth * (1.0 - w);

    return Stack(
      children: <Widget>[
        Positioned(
          top: 0,
          left: 0,
          right: right,
          bottom: 0,
          child: widget.view1,
        ),
        Positioned(
          top: 0,
          left: left,
          right: 0,
          bottom: 0,
          child: widget.view2,
        ),
        Positioned(
          top: 0,
          left: left - widget.gripSize / 2.0,
          right: right - widget.gripSize / 2.0,
          bottom: 0,
          child: GestureDetector(
            behavior: HitTestBehavior.translucent,
            onVerticalDragUpdate: (detail) {
              final RenderBox container = context.findRenderObject() as RenderBox;
              final pos = container.globalToLocal(detail.globalPosition);
              if (pos.dx > widget.positionLimit &&
                  pos.dx < (container.size.width - widget.positionLimit)) {
                weight.value = pos.dx / container.size.width;
              }
            },
            child: Container(color: widget.gripColor),
          ),
        ),
      ],
    );
  }
}

enum SplitViewMode {
  Vertical,
  Horizontal,
}
